#include <avr/io.h>

/*
 ** getFuseBits
 *
 * Read fuse or lock bits.
 *
 * On entry:   r24 contains the index of the desired fuse/lock bit register
 *                  0 : low fuse byte
 *                  1 : lock bits
 *                  2 : high fuse byte
 *                  3 : extended fuse byte
 *
 * On exit:    r24 contains the requested value or zero for an invalid request
 *
 * C-callable: uint8_t getFuseBits(uint8_t idx)
 *
 * Modifies:   r24, r25, r30, r31
 *
 */
	.global getFuseBits

getFuseBits:
#define idx			r24
#define sreg_save	r25

	cpi		idx, 4
	brlo	readFuseLock

	// bad index
	ldi		r24, 0
	ret

readFuseLock:
	// move the index to r31:30
	mov		r30, r24
	clr		r31

	// save the status register, disable interrupts
	in		sreg_save, 0x3f
	cli

	// set the enabling bits, read the value
	ldi		r24, _BV(SELFPRGEN) | _BV(BLBSET)
	sts		SPMCSR, r24
	lpm		r24, Z

	// restore the previous interrupt enable state
	out		0x3f, sreg_save

	ret
#undef idx
#undef sreg_save

	.end
